#
# Spec file for packaging OFED kernel modules (and corresponding
# header files).
#

%define kversion 2.6.18-128.4.1.0.1.el5

Name:		ofa-%{kversion}
Summary:	OpenFabrics Infiniband Kernel Modules
Prereq:		/sbin/ldconfig
Version:	1.4.2
Release:	10
License:	GPL/BSD
Url:		http://www.openfabrics.org/
Group:		System Environment/Base
Source:		ofa_kernel-%{version}.tgz

Patch0:		less-verbose-build.patch
Patch1:		ib-bonding-configure.patch
Patch2:		ib-bonding-source.patch
Patch3:		mlx-msix-retry-fewer-vectors.patch

# RDS patches
Patch10:        0001-RDS-Fix-potential-race-around-rds_i-bw-_allocation.patch
Patch11:        0002-RDS-Fix-panic-on-unload.patch
Patch21:        0003-RDS-Do-not-send-congestion-updates-to-loopback-conn.patch
Patch22:        0004-RDS-IB-Move-recv-processing-to-a-tasklet.patch
Patch32:        0005-RDS-IW-Fix-broken-test-of-atomic_add_unless.patch
Patch33:        0006-RDS-Fix-congestion-issues-for-loopback.patch
Patch43:        0007-RDS-Fix-send-locking-issue.patch
Patch53:        0008-qlgc_vnic-Fix-typo-in-header-guard.patch
Patch54:        0009-qlgc_vnic-Rename-struct-mc_info-to-vnic_mc_info-to.patch
Patch64:        0010-RDS-Workaround-for-in-use-MRs-on-close-causing-cras.patch
Patch65:        0011-RDS-Turn-down-alarming-reconnect-messages.patch
Patch75:        0012-IPoIB-Clear-ipoib_neigh.dgid-in-ipoib_neigh_alloc.patch

BuildRoot:	%{?build_root:%{build_root}}%{!?build_root:/var/tmp/OFED}
Vendor:		OpenFabrics
Requires:       kernel = %{kversion}
BuildRequires:  kernel-devel = %{kversion}
#BuildRequires:	sysfsutils-devel

%description
Infiniband kernel modules, providing the core functionality including
the IB verbs layer, drivers for a variety of adapters, management
modules, and upper layer protocols. This package is compiled for
the default kernel.

# Turn ons and offs
%define builddef        1
%define buildpae        0
%define buildxen        0
%define builddebug      0

%ifarch i686
%define karch		i686
%define buildpae        1
%define buildxen        1
%define builddebug      1
%endif

%ifarch x86_64
%define karch		x86_64
%define buildxen        1
%define builddebug      1
%endif

%if %{buildpae}
%package -n %{name}PAE
Summary:        OpenFabrics Infiniband Kernel Modules for the PAE kernel
Group:          System Environment/Base
Requires:       kernel-PAE = %{kversion}
BuildRequires:  kernel-PAE-devel = %{kversion}

%description -n %{name}PAE
Infiniband kernel modules, providing the core functionality including
the IB verbs layer, drivers for a variety of adapters, management
modules, and upper layer protocols. This package is compiled for 
the PAE kernel.
%endif

%if %{buildxen}
%package -n %{name}xen
Summary:        OpenFabrics Infiniband Kernel Modules for the xen kernel
Group:          System Environment/Base
Requires:       kernel-xen = %{kversion}
BuildRequires:  kernel-xen-devel = %{kversion}

%description -n %{name}xen
Infiniband kernel modules, providing the core functionality including
the IB verbs layer, drivers for a variety of adapters, management
modules, and upper layer protocols. This package is compiled for
the xen kernel.
%endif

%if %{builddebug}
%package -n %{name}debug
Summary:        OpenFabrics Infiniband Kernel Modules for the debug kernel
Group:          System Environment/Base
Requires:       kernel-debug = %{kversion}
BuildRequires:  kernel-debug-devel = %{kversion}

%description -n %{name}debug
Infiniband kernel modules, providing the core functionality including
the IB verbs layer, drivers for a variety of adapters, management
modules, and upper layer protocols. This package is compiled for
the debug kernel.
%endif


%prep

%setup -q -n ofa_kernel-%{version}
%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1

# RDS patches
%patch10 -p1
%patch11 -p1
%patch21 -p1
%patch22 -p1
%patch32 -p1
%patch33 -p1
%patch43 -p1
%patch53 -p1
%patch54 -p1
%patch64 -p1
%patch65 -p1
%patch75 -p1

sh ./configure.bonding  --kernel-version %{kversion}

%build
rm -rf $RPM_BUILD_ROOT

local_configure()
{
# This freakin stupid rpmbuild 4.4.2 doesn't seem to understand
# line continuations via backslash :-(
# Hence this gobbledigook
options="
 --kernel-version="$1"
 --modules-dir=/lib/modules/"$1"
 --kernel-sources=/usr/src/kernels/"$1"-"$2"
 --without-quilt
 --with-bonding-mod
 --with-rds-mod
 --with-core-mod
 --with-ipoib-mod
 --with-ipoib-cm
 --with-bonding-mod
 --with-user_mad-mod
 --with-user_access-mod
 --with-addr_trans-mod
 --with-madeye-mod
 --with-mthca-mod
 --with-mlx4-mod
 --with-cxgb3-mod
 --with-qlgc_vnic-mod
 --with-nes-mod
 --with-sdp-mod
 --with-srp-mod
 --with-srp-target-mod
"
./configure $options
}

%if %{builddef}
local_configure "%{kversion}" "%{karch}"

make
make install_kernel DESTDIR=$RPM_BUILD_ROOT # MODULES_DIR=%{LIB_MOD_DIR}

MOD_DIR=$RPM_BUILD_ROOT/lib/modules/%{kversion}
modules_dir=`ls -d $MOD_DIR`
if [ -d $modules_dir/kernel ]; then
   mv $modules_dir/kernel $modules_dir/updates
fi

make clean_kernel

# Build the list of files to include. It's easier to do it this
# way that trying to figure out exactly which kernel modules get
# built and installed.
(
	cd $RPM_BUILD_ROOT
	find lib/modules/%{kversion}/updates -type f | sed 's:^:/:'
) > files-def
%endif

%if %{buildpae}
local_configure "%{kversion}PAE" "%{karch}"

make
make install_kernel DESTDIR=$RPM_BUILD_ROOT # MODULES_DIR=%{LIB_MOD_DIR}

MOD_DIR=$RPM_BUILD_ROOT/lib/modules/%{kversion}PAE
modules_dir=`ls -d $MOD_DIR`
if [ -d $modules_dir/kernel ]; then
   mv $modules_dir/kernel $modules_dir/updates
fi

make clean_kernel

# Build the list of files to include. It's easier to do it this
# way that trying to figure out exactly which kernel modules get
# built and installed.
(
	cd $RPM_BUILD_ROOT
	find lib/modules/%{kversion}PAE/updates -type f | sed 's:^:/:'
) > files-pae

%endif

%if %{buildxen}
local_configure "%{kversion}xen" "%{karch}"

make
make install_kernel DESTDIR=$RPM_BUILD_ROOT # MODULES_DIR=%{LIB_MOD_DIR}

MOD_DIR=$RPM_BUILD_ROOT/lib/modules/%{kversion}xen
modules_dir=`ls -d $MOD_DIR`
if [ -d $modules_dir/kernel ]; then
   mv $modules_dir/kernel $modules_dir/updates
fi

make clean_kernel

# Build the list of files to include. It's easier to do it this
# way that trying to figure out exactly which kernel modules get
# built and installed.
(
	cd $RPM_BUILD_ROOT
	find lib/modules/%{kversion}xen/updates -type f | sed 's:^:/:'
) > files-xen

%endif

%if %{builddebug}
local_configure "%{kversion}debug" "%{karch}"

make
make install_kernel DESTDIR=$RPM_BUILD_ROOT # MODULES_DIR=%{LIB_MOD_DIR}

MOD_DIR=$RPM_BUILD_ROOT/lib/modules/%{kversion}debug
modules_dir=`ls -d $MOD_DIR`
if [ -d $modules_dir/kernel ]; then
   mv $modules_dir/kernel $modules_dir/updates
fi

make clean_kernel

# Build the list of files to include. It's easier to do it this
# way that trying to figure out exactly which kernel modules get
# built and installed.
(
	cd $RPM_BUILD_ROOT
	find lib/modules/%{kversion}debug/updates -type f | sed 's:^:/:'
) > files-debug

%endif

%install
# Do nothing here

%clean
rm -rf "$RPM_BUILD_ROOT"

%if %{builddef}
%post
/sbin/depmod -ae %{kversion}

%postun
/sbin/depmod -ae %{kversion}

%files -f files-def
%defattr(-,root,root)
%endif

%if %{buildpae}
%post -n %{name}PAE
/sbin/depmod -ae %{kversion}PAE

%postun -n %{name}PAE
/sbin/depmod -ae %{kversion}PAE

%files -f files-pae -n %{name}PAE
%defattr(-,root,root)
%endif

%if %{buildxen}
%post -n %{name}xen
/sbin/depmod -ae %{kversion}xen

%postun -n %{name}xen
/sbin/depmod -ae %{kversion}xen

%files -f files-xen -n %{name}xen
%defattr(-,root,root)
%endif

%if %{builddebug}
%post -n %{name}debug
/sbin/depmod -ae %{kversion}debug

%postun -n %{name}debug
/sbin/depmod -ae %{kversion}debug

%files -f files-debug -n %{name}debug
%defattr(-,root,root)
%endif

%changelog
* Tue Feb 16 2010 Andy Grover <andy.grover@oracle.com> - 1.4.2-10
- Add patches
  0012-IPoIB-Clear-ipoib_neigh.dgid-in-ipoib_neigh_alloc.patch

* Tue Feb 16 2010 Andy Grover <andy.grover@oracle.com> - 1.4.2-9
- Add patches
  0010-RDS-Workaround-for-in-use-MRs-on-close-causing-cras.patch
  0011-RDS-Turn-down-alarming-reconnect-messages.patch

* Mon Feb 01 2010 Andy Grover <andy.grover@oracle.com> - 1.4.2-8
- Add patches
  0008-qlgc_vnic-Fix-typo-in-header-guard.patch
  0009-qlgc_vnic-Rename-struct-mc_info-to-vnic_mc_info-to.patch

* Thu Jan 28 2010 Andy Grover <andy.grover@oracle.com> - 1.4.2-7
- Add patches
  0007-RDS-Fix-send-locking-issue.patch

* Thu Jan 21 2010 Andy Grover <andy.grover@oracle.com> - 1.4.2-6
- Add patches
  0005-RDS-IW-Fix-broken-test-of-atomic_add_unless.patch
  0006-RDS-Fix-congestion-issues-for-loopback.patch

* Tue Nov 17 2009 Andy Grover <andy.grover@oracle.com> - 1.4.2-5
- Remove patches
  0005-RDS-Ask-ib_create_cq-to-attach-to-least-used-vector.patch
- Remove iser from build, due to weird crypto_api issue

* Thu Sep 24 2009 Andy Grover <andy.grover@oracle.com> - 1.4.2-4
- Add patches
  mlx-msix-retry-fewer-vectors.patch
  0005-RDS-Ask-ib_create_cq-to-attach-to-least-used-vector.patch

* Wed Sep 23 2009 Andy Grover <andy.grover@oracle.com> - 1.4.2-3
- Add patches
  0003-RDS-Do-not-send-congestion-updates-to-loopback-conn.patch
  0004-RDS-IB-Move-recv-processing-to-a-tasklet.patch

* Fri Aug 14 2009 Andy Grover <andy.grover@oracle.com> - 1.4.2-2
- Add patches
  0001-RDS-Fix-potential-race-around-rds_i-bw-_allocation.patch
  0002-RDS-Fix-panic-on-unload.patch

* Wed Aug 12 2009 Andy Grover <andy.grover@oracle.com> - 1.4.2-1
- Update OFED tarball to 1.4.2 release
- Remove patches
  0001-RDS-Fix-completion-notifications-on-blocking-socket.patch
  0002-RDS-IB-Always-use-PAGE_SIZE-for-FMR-page-size.patch
  0003-RDS-IW-Remove-page_shift-variable-from-iwarp-transp.patch
  0004-RDS-IW-Remove-dead-code.patch

* Mon Jun 15 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-6.20090528
- Add patches
  0001-RDS-Fix-completion-notifications-on-blocking-socket.patch
  0002-RDS-IB-Always-use-PAGE_SIZE-for-FMR-page-size.patch
  0003-RDS-IW-Remove-page_shift-variable-from-iwarp-transp.patch
  0004-RDS-IW-Remove-dead-code.patch

* Thu May 28 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-5.20090528
- Update tarball to 1.4.1 release
- Remove patches
  0001-RDS-IB-Improve-RDS-protocol-version-checking.patch
  0002-RDS-IB-Handle-connections-using-RDS-3.0-wire-protoc.patch
  0003-RDS-IB-Fix-printk-to-indicate-remote-IP-not-local.patch
  0004-RDS-RDMA-Fix-cut-n-paste-errors-in-printks-in-rdma_.patch
  0005-RDS-IB-Rename-byte_len-to-data_len-to-enhance-reada.patch
  0006-RDS-Don-t-set-c_version-in-__rds_conn_create.patch
  0007-RDS-IB-Move-tx-rx-ring-init-and-refill-to-later.patch
  0008-RDS-IB-Disable-flow-control-in-sysctl-and-explain-w.patch
  0009-RDS-IB-Drop-connection-when-a-fatal-QP-event-is-rec.patch

* Tue May 12 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-4.20090430
- Add patches
  0001-RDS-IB-Improve-RDS-protocol-version-checking.patch
  0002-RDS-IB-Handle-connections-using-RDS-3.0-wire-protoc.patch
  0003-RDS-IB-Fix-printk-to-indicate-remote-IP-not-local.patch
  0004-RDS-RDMA-Fix-cut-n-paste-errors-in-printks-in-rdma_.patch
  0005-RDS-IB-Rename-byte_len-to-data_len-to-enhance-reada.patch
  0006-RDS-Don-t-set-c_version-in-__rds_conn_create.patch
  0007-RDS-IB-Move-tx-rx-ring-init-and-refill-to-later.patch
  0008-RDS-IB-Disable-flow-control-in-sysctl-and-explain-w.patch
  0009-RDS-IB-Drop-connection-when-a-fatal-QP-event-is-rec.patch

* Wed May 4 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-3.20090430
- Update tarball to 1.4.1-rc4
- Update bonding patches to ib-bonding-0.9.0 my_release 39
- Apply all bonding patches into ip-bonding-source
- Remove patches
  0001-ERR_PTR-dereference-in-net-rds-iw.c.patch
  0002-ERR_PTR-dereference-in-net-rds-ib.c.patch
  0003-RDS-Set-retry_count-to-2-and-make-modifiable-via-mo.patch

* Wed Apr 29 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-2.20090406
- Update tarball to 1.4.1-rc3
- Add patches
  0001-ERR_PTR-dereference-in-net-rds-iw.c.patch
  0002-ERR_PTR-dereference-in-net-rds-ib.c.patch
  0003-RDS-Set-retry_count-to-2-and-make-modifiable-via-mo.patch
- Remove patches
  0001-RDS-Fix-m_rs_lock-deadlock.patch
  0002-RDS-Fix-race-causing-double-list-del.patch
  0003-RDS-IW-IB-Set-recv-ring-low-water-mark-to-1-2-full.patch
  0004-RDS-Correct-some-iw-references-in-rdma_transport.c.patch
  0005-RDS-IW-IB-Set-the-RDS_LL_SEND_FULL-bit-when-we-re-t.patch
  0006-RDS-IW-IB-Allow-max-credit-advertise-window.patch
  0007-RDS-Fix-ordering-in-a-conditional.patch
  0008-RDS-Establish-connection-before-parsing-CMSGs.patch

* Mon Mar 16 2009 Andy Grover <andy.grover@oracle.com> - 1.4.1-1.20090304
- Initial packaging of OFED 1.4 for OEL5
- Re-enable building all OFED modules; required for successful compilation
- Disable bonding patches
- Add patches
  0001-RDS-Fix-m_rs_lock-deadlock.patch
  0002-RDS-Fix-race-causing-double-list-del.patch
  0003-RDS-IW-IB-Set-recv-ring-low-water-mark-to-1-2-full.patch
  0004-RDS-Correct-some-iw-references-in-rdma_transport.c.patch
  0005-RDS-IW-IB-Set-the-RDS_LL_SEND_FULL-bit-when-we-re-t.patch
  0006-RDS-IW-IB-Allow-max-credit-advertise-window.patch
  0007-RDS-Fix-ordering-in-a-conditional.patch
  0008-RDS-Establish-connection-before-parsing-CMSGs.patch

* Fri Jan 16 2009 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.2-1.20090106
- Updated the tarball to OFED-1.3.2 (OFED-1.3.2-20090106-0158) for OEL5U3
- Add patches from [andy.grover@oracle.com]
  0042-RDS-Fix-connections-to-127.x.x.x.patch
  0043-RDS-Set-retry_count-to-2-and-make-modifiable-via-mo.patch

* Wed Dec 10 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.2-1.20081117
- Updated the tarball to OFED-1.3.2 (OFED-1.3.2-20081117-0511) for OEL5U3
  removing all the RDS patches
- Update to build modules for debug kernel also

* Mon Nov 10 2008 Andy Grover <andy.grover@oracle.com> - 1.3.1-6.20080603
- Remove patch
  0023-RDS-Fix-compilation-error.patch
- Add patches
  0023-RDS-Fix-spin-between-poll-and-sendmsg-each-retu.patch
  0024-RDS-Fix-compilation-issue-due-to-EVENT_ADDR_CHANGE.patch
  0025-RDS-Fix-send-ring-leak.patch
  0026-RDS-Fix-race-between-incoming-packet-and-socket-clo.patch
  0027-RDS-Don-t-send-congestion-maps-over-IB-loopback.patch
  0028-RDS-rewrite-connection-and-ipaddr-list-handling-for.patch
  0029-RDS-Fix-a-congestion-hang-under-stress.patch
  0030-RDS-Remove-TCP-transport.patch

* Thu Sep 04 2008 Andy Grover <andy.grover@oracle.com> - 1.3.1-5.20080603
- Uncomment Patch12
  0009-RDS-Wake-up-send-worker-after-TX-queue-full-conditi.patch
  and add patches
  0015-RDS-Fix-compilation-issue-on-PPC64-and-IA64.patch
  0016-net-rds-Add-support-for-RDMA_CM_EVENT_ADDR_CHANGE-e.patch
  0017-RDS-Fix-race-in-rds_send_xmit.patch
  0018-RDS-IB-silence-harmless-kernel-message.patch
  0019-RDS-IB-Increase-FMR-pool-size-to-4k.patch
  0020-RDS-IB-remove-error-message-on-connection-race.patch
  0021-RDS-IB-Don-t-count-congestion-messages-for-determin.patch
  0022-RDS-IB-Always-set-i_ack_next-in-rds_ib_set_ack.patch
  0023-RDS-Fix-compilation-error.patch
   
* Thu Jun 26 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.1-4.20080603
- Comment out/remove this patch:
  0009-RDS-Wake-up-send-worker-after-TX-queue-full-conditi.patch.
  Enabled configure to build srpt module.

* Fri Jun 20 2008 Van Okamura <van.okamura@oracle.com> - 1.3.1-3.20080603
- Use updated patches from [olaf.kirch@oracle.com]:
  0001-RDS-Clarify-comment-on-rds_poll.patch
  0002-RDS-introduce-rds_send_get_message.patch
  0003-RDS-Track-whether-RDMA-S-G-list-is-DMA-mapped.patch
  0004-RDS-Another-fix-for-RDMA-signalling.patch
  0005-RDS-Use-vmalloc-in-rds_ib_setup_qp.patch
  0006-RDS-increase-size-of-stats-counters.patch
  0007-RDS-Expose-info-on-IB-connections.patch
  0008-RDS-Extend-IB-connection-info-to-include-settings.patch
  0009-RDS-Wake-up-send-worker-after-TX-queue-full-conditi.patch
  0010-RDS-Send-notification-for-cancelled-RDMA-ops.patch
  0011-RDS-Add-mr_pool_depleted-stats-counter.patch
  0012-RDS-rds-info-k-should-display-inode-number.patch
  0013-RDS-Remote-spurious-variable-name-from-stats.patch
  0014-RDS-Increase-default-FMR-pool-size-to-4K.patch

* Wed Jun 11 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.1-2.20080603
- Added these RDS patches from [olaf.kirch@oracle.com]
    rds-poll-comment.patch, rds-send-find-message.patch, rds-rdma-tracked-mapped.patch,
    rdma-error-propagation.patch, rds-ib-ring-vmalloc.patch, rds-bigger-byte-counters.patch
    rds-ib-connection-info.patch, rds-ib-tx-pressure.patch, rds-cancel-notify.patch
    rds-get-mr-fail-stats.patch, and rds-sock-info-inum.patch
- Replaced the tarball from the OFED's ofa-kernel src rpm of OFED-1.3.1 build

* Tue Jun 03 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.1-2.20080529
- Removed the ib-bonding.patch based on ib-bonding-0.9.0-24 package from OFED
  and added ib-bonding-configure.patch and ib-bonding-source.patch based on
  ib-bonding-0.9.0-25 from OFED-1.3.1-rc3
- Added kernel-addons-backport-2.6.18-EL5.1.patch to fix build of ib-bonding 
  module for EL5.1 kernels
- Removed rhel5u2-build.patch as the tarball in ofa-kernel src rpm from
  OFD-1.3.1-rc3 supports EL5.2 kernels

* Tue Jun 02 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3.1-1.20080529
- Replaced the tarball from the OFED's ofa-kernel src rpm of OFED-1.3.1-rc3 build
- Added patch rdma-fix-failure.patch [richard.frank@oracle.com]
- Commented ib-bonding.patch 

* Tue May 20 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3-1.20080513
- Added ib-bonding.patch based on ib-bonding-0.9.0-24 package from OFED-1.3.1
  to package the ib bonding kernel module in ofa 

* Wed May 14 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3-1.20080513
- Replaced the tarball from the OFED's ofa-kernel src rpm of 20080513 build
- Update rhel5u2-build.patch based on this new tarball
- Update less-verbose-build.patch

* Mon May 08 2008 Srinivas Maturi <srinivas.maturi@oracle.com> - 1.3-0.2
- Added patch rhel5u2-build.patch to build in (RH)EL5 U2
- Add support to build for all kernel flavors for i386/x86_64 arch
- Add postun section and change package name

* Mon Apr 28 2008 Olaf Kirch <olaf.kirch@oracle.com>
- Update specfile enabling more modules to build

* Mon Apr 14 2008 Olaf Kirch <olaf.kirch@oracle.com>
 - Initial packaging of OFED 1.3 for OEL5

